@@ -28,7 +28,7 @@ from typing import List
 # Third party libraries
 import aacgmv2
 
-from pydarn import (PyDARNColormaps, Fan, plot_exceptions, citing_warning, 
+from pydarn import (PyDARNColormaps, Fan, plot_exceptions, citing_warning,
     standard_warning_format)
 
 warnings.formatwarning = standard_warning_format
@@ -49,13 +49,12 @@ class Grid():
     @classmethod
     def plot_grid(cls, dmap_data: List[dict], record: int = 0,
                   start_time: dt.datetime = None, time_delta: int = 1,
-                  ax=None, fov: bool = True, parameter: str = 'vel',
-                  lowlat: int = 30, cmap: str = None, zmin: int = None,
+                  ax=None, parameter: str = 'vel',
+                  cmap: str = None, zmin: int = None,
                   zmax: int = None, colorbar: bool = True,
                   colorbar_label: str = '', title: str = '',
                   len_factor: float = 150.0, ref_vector: int = 300,
-                  fov_color: str = None, line_color: str = 'black',
-                  radar_location: bool = True, radar_label: bool = False):
+                  **kwargs):
         """
         Plots a radar's gridded vectors from a GRID file
 
@@ -82,12 +81,6 @@ class Grid():
             parameter: str
                 Key name indicating which parameter to plot.
                 Default: vel (Velocity). Alternatives: 'pwr', 'wdt'
-            lowlat: int
-                Lower AACGM latitude boundary for the polar plot
-                Default: 50
-            fov: bool
-                Set to false to not plot the outline of the FOV
-                Default: True
             cmap: matplotlib.cm
                 matplotlib colour map
                 https://matplotlib.org/tutorials/colors/colormaps.html
@@ -116,19 +109,8 @@ class Grid():
             ref_vector: int
                 Velocity value to be used for the reference vector, in m/s
                 Default: 300
-            fov_color: str
-                Field-of-view fill color
-                default: '' - transparent
-            line_color: str
-                Colour of the field-of-view fan lines
-                default: black
-            radar_label: bool
-                place the radar abbreviation on the plot
-                default: False
-            radar_location: bool
-                place the radar location represented as a dot
-                default: True
-
+            kwargs: key=value
+                uses the parameters for plot_fov and projections.axis
         See Also
         --------
         plot_fov - plots the field of view found in fan.py
@@ -155,12 +137,12 @@ class Grid():
         # Find the record corresponding to the start time
         if start_time is not None:
             for record in range(len(dmap_data)):
-                dtime = dt.datetime(dmap_data[record]['start.year'],
+                date = dt.datetime(dmap_data[record]['start.year'],
                                     dmap_data[record]['start.month'],
                                     dmap_data[record]['start.day'],
                                     dmap_data[record]['start.hour'],
                                     dmap_data[record]['start.minute'])
-                time_diff = dtime - start_time
+                time_diff = date - start_time
                 if time_diff.seconds/60 <= time_delta:
                     break
             if time_diff.seconds/60 > time_delta:
@@ -168,7 +150,7 @@ class Grid():
                                                        start_time=start_time)
         else:
             record = 0
-            dtime = dt.datetime(dmap_data[record]['start.year'],
+            date = dt.datetime(dmap_data[record]['start.year'],
                                 dmap_data[record]['start.month'],
                                 dmap_data[record]['start.day'],
                                 dmap_data[record]['start.hour'],
@@ -178,20 +160,14 @@ class Grid():
             warnings.simplefilter("ignore")
             for stid in dmap_data[record]['stid']:
                 _, aacgm_lons, _, _, ax =\
-                        Fan.plot_fov(stid, dtime,
-                                     boundary=fov,
-                                     lowlat=lowlat,
-                                     line_color=line_color,
-                                     fov_color=fov_color,
-                                     radar_location=radar_location,
-                                     radar_label=radar_label,
-                                     ax=ax)
+                        Fan.plot_fov(stid, date,
+                                     ax=ax, **kwargs)
                 data_lons = dmap_data[record]['vector.mlon']
                 data_lats = dmap_data[record]['vector.mlat']
 
                 # Hold the beam positions
                 shifted_mlts = aacgm_lons[0, 0] - \
-                    (aacgmv2.convert_mlt(aacgm_lons[0, 0], dtime) * 15)
+                    (aacgmv2.convert_mlt(aacgm_lons[0, 0], date) * 15)
                 shifted_lons = data_lons - shifted_mlts
                 thetas = np.radians(shifted_lons)
                 rs = data_lats
@@ -288,11 +264,11 @@ class Grid():
         if title == '':
             title = "{year}-{month}-{day} {start_hour}:{start_minute} -"\
                 " {end_hour}:{end_minute}"\
-                    "".format(year=dtime.year,
-                              month=str(dtime.month).zfill(2),
-                              day=str(dtime.day).zfill(2),
-                              start_hour=str(dtime.hour).zfill(2),
-                              start_minute=str(dtime.minute).zfill(2),
+                    "".format(year=date.year,
+                              month=str(date.month).zfill(2),
+                              day=str(date.day).zfill(2),
+                              start_hour=str(date.hour).zfill(2),
+                              start_minute=str(date.minute).zfill(2),
                               end_hour=str(dmap_data[record]['end.hour']).
                               zfill(2),
                               end_minute=str(dmap_data[record]['end.minute']).
