@@ -5,7 +5,8 @@
 #
 # Modifications:
 # 2021-05-12 Francis Tholley added gate2grounscatter to range-time plots
-#
+# 2021-06-18 Marina Schmidt (SuperDARN Canada) fixed ground scatter colour bug
+# 2021-07-06 Carley Martin added keyword to aid in rounding start times
 # Disclaimer:
 # pyDARN is under the LGPL v3 license found in the root directory LICENSE.md
 # Everyone is permitted to copy and distribute verbatim copies of this license
@@ -20,6 +21,7 @@
 """
 Range-Time Parameter (aka Intensity) plots
 """
+import copy
 import matplotlib.pyplot as plt
 import numpy as np
 import warnings
@@ -72,11 +74,11 @@ class RTP():
                         colorbar: plt.colorbar = None, ymin: int = None,
                         ymax: int = None, yspacing: int = 200,
                         coords: object = Coords.SLANT_RANGE,
-                        reflection_height: float = 250.0,
                         colorbar_label: str = '',
                         norm=colors.Normalize, cmap: str = None,
                         filter_settings: dict = {},
-                        date_fmt: str = '%y/%m/%d\n %H:%M', **kwargs):
+                        date_fmt: str = '%y/%m/%d\n %H:%M',
+                        round_start: bool=True, **kwargs):
         """
         Plots a range-time parameter plot of the given
         field name in the dmap_data
@@ -128,10 +130,6 @@ class RTP():
         coords: Coords
             set the y-axis to a desired coordinate system
             Default: Coords.SLANT_RANGE
-        reflection_height: float
-            set the ionosphere virtual reflection height
-            this only applies to Coords.GROUND_SCATTER_MAPPED_RANGE
-            Default: 250 (km)
         norm: matplotlib.colors.Normalization object
             This object use dependency injection to use any normalization
             method with the zmin and zmax.
@@ -180,10 +178,13 @@ class RTP():
                 dictionary that contains the key parameter names and the value
                 to compare against. Will filter out data sections
                 that is does not equal the value.
+        round_start: bool=True
+                option to round the start time to give tick at start of xaxis
+                Set True to round, set False to plot from start of data.
+                Default: True
         kwargs:
-            key names of variable settings of pcolormesh:
-            https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.pcolormesh.html
-
+            used for other methods in pyDARN
+                - reflection_height
         Raises
         ------
         UnknownParameterError
@@ -355,19 +356,19 @@ class RTP():
                                      start_time=start_time,
                                      end_time=end_time,
                                      opt_beam_num=cls.dmap_data[0]['bmnum'])
-        if coords is Coords.SLANT_RANGE:
+        if coords is Coords.SLANT_RANGE or\
+           coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
             # Get rxrise from hardware files (consistent with RST)
             rxrise = SuperDARNRadars.radars[cls.dmap_data[0]['stid']]\
                                     .hardware_info.rx_rise_time
-            y = gate2slant(cls.dmap_data[0], y_max, rxrise=rxrise)
-        elif coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
-            rxrise = SuperDARNRadars.radars[cls.dmap_data[0]['stid']]\
-                                    .hardware_info.rx_rise_time
-            y = gate2slant(cls.dmap_data[0], y_max, rxrise=rxrise)
-            y = gate2GroundScatter(y, reflection_height)
-            y0inx = np.min(np.where(np.isfinite(y))[0])
-            y = y[y0inx:]
-            z = z[:,y0inx:]
+            y = gate2slant(cls.dmap_data[0]['frang'], cls.dmap_data[0]['rsep'],
+                           rxrise, nrang=y_max)
+            if coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
+                y = gate2GroundScatter(y, **kwargs)
+                y0inx = np.min(np.where(np.isfinite(y))[0])
+                y = y[y0inx:]
+                z = z[:, y0inx:]
+
         time_axis, y_axis = np.meshgrid(x, y)
         z_data = np.ma.masked_where(np.isnan(z.T), z.T)
         Default = {'noise.sky': (1e0, 1e5),
@@ -390,36 +391,36 @@ class RTP():
                           " options".format(zmax))
         norm = norm(zmin, zmax)
         if isinstance(cmap, str):
-            cmap = cm.get_cmap(cmap)
+            cmap = colors.Colormap(cmap)
         else:
-            cmaps = {'p_l': 'plasma',
+            # need to do this as matplotlib 3.5 will
+            # not all direct mutations of the object
+            cmaps = {'p_l': copy.copy(cm.get_cmap('plasma')),
                      'v': PyDARNColormaps.PYDARN_VELOCITY,
                      'w_l': PyDARNColormaps.PYDARN_VIRIDIS,
                      'elv': PyDARNColormaps.PYDARN}
             cmap = cmaps[parameter]
 
-        if isinstance(groundscatter, str):
-            cmap.set_under(groundscatter, 1.0)
-        elif groundscatter:
-            cmap.set_under('grey', 1.0)
-
         # set the background color, this needs to happen to avoid
         # the overlapping problem that occurs
         cmap.set_bad(color=background, alpha=1.)
         # plot!
         im = ax.pcolormesh(time_axis, y_axis, z_data, lw=0.01,
                            cmap=cmap, norm=norm, **kwargs)
+
+        if isinstance(groundscatter, str):
+            ground_scatter = np.ma.masked_where(z_data != -1000000, z_data)
+            gs_color = colors.ListedColormap([groundscatter])
+            ax.pcolormesh(time_axis, y_axis, ground_scatter, lw=0.01,
+                          cmap=gs_color, norm=norm, **kwargs)
+
+        elif groundscatter:
+            ground_scatter = np.ma.masked_where(z_data != -1000000, z_data)
+            gs_color = colors.ListedColormap(['grey'])
+            ax.pcolormesh(time_axis, y_axis, ground_scatter, lw=0.01,
+                          cmap=gs_color, norm=norm, **kwargs)
+
         # setup some standard axis information
-        # Upon request of Daniel Billet and others, I am rounding
-        # the time down so the plotting x-axis will show the origin
-        # time label
-        # TODO: may need to be its own function
-        rounded_down_start_time = x[0] -\
-            timedelta(minutes=x[0].minute % 15,
-                      seconds=x[0].second,
-                      microseconds=x[0].microsecond)
-        ax.set_xlim([rounded_down_start_time, x[-1]])
-        ax.xaxis.set_major_formatter(dates.DateFormatter(date_fmt))
         if ymax is None:
             ymax = np.max(y)
 
@@ -428,7 +429,8 @@ class RTP():
 
         ax.set_ylim(ymin, ymax)
 
-        if coords is Coords.SLANT_RANGE or coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
+        if coords is Coords.SLANT_RANGE or\
+           coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
             ax.yaxis.set_ticks(np.arange(np.ceil(ymin/100.0)*100,
                                          ymax+1, yspacing))
         else:
@@ -444,9 +446,39 @@ class RTP():
             tick_interval = 30
         else:
             tick_interval = 1
-        ax.xaxis.set_minor_locator(dates.MinuteLocator(interval=tick_interval))
+        # byminute keyword makes sure that the ticks are situated at
+        # the minute or half hour marks, rather than at a set interval
+        ax.xaxis.set_minor_locator(
+            dates.MinuteLocator(byminute=range(0,60,tick_interval)))
 
-        if coords is Coords.SLANT_RANGE or coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
+        # Upon request of Daniel Billet and others, I am rounding
+        # the time down so the plotting x-axis will show the origin
+        # time label
+        # Updated to give option to round down and make sure
+        # rounding to same frequency as plot axis ticks if less than 1 hour
+        if round_start:
+            major_locator, _ = plt.xticks()
+            dt = dates.num2date(major_locator[1]) -\
+                dates.num2date(major_locator[0])
+            tick_sep = dt.seconds//60
+            if tick_sep > 0:
+                rounded_down_start_time = x[0] -\
+                    timedelta(minutes=x[0].minute % tick_sep,
+                          seconds=x[0].second,
+                          microseconds=x[0].microsecond)
+            else:
+                rounded_down_start_time = x[0] -\
+                    timedelta(minutes=x[0].minute % 15,
+                          seconds=x[0].second,
+                          microseconds=x[0].microsecond)
+        else:
+            rounded_down_start_time = x[0]
+
+        ax.set_xlim([rounded_down_start_time, x[-1]])
+        ax.xaxis.set_major_formatter(dates.DateFormatter(date_fmt))
+
+        if coords is Coords.SLANT_RANGE or\
+           coords is Coords.GROUND_SCATTER_MAPPED_RANGE:
             ax.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))
         else:
             ax.yaxis.set_minor_locator(ticker.MultipleLocator(5))
@@ -461,8 +493,10 @@ class RTP():
                     if isinstance(norm, colors.LogNorm):
                         cb = ax.figure.colorbar(im, ax=ax, extend='both')
                     else:
-                        locator = ticker.MaxNLocator(symmetric=True, min_n_ticks=3,
-                                                     integer=True, nbins='auto')
+                        locator = ticker.MaxNLocator(symmetric=True,
+                                                     min_n_ticks=3,
+                                                     integer=True,
+                                                     nbins='auto')
                         ticks = locator.tick_values(vmin=zmin, vmax=zmax)
                         cb = ax.figure.colorbar(im, ax=ax, extend='both',
                                                 ticks=ticks)
@@ -479,11 +513,13 @@ class RTP():
     @classmethod
     def plot_time_series(cls, dmap_data: List[dict],
                          parameter: str = 'tfreq', beam_num: int = 0,
-                         ax=None, start_time: datetime = None,
+                         ax=None, gate: int = 0, start_time: datetime = None,
                          end_time: datetime = None,
                          date_fmt: str = '%y/%m/%d\n %H:%M',
                          channel='all', scale: str = 'linear',
-                         cp_name: bool = True, **kwargs):
+                         cp_name: bool = True, color: str = 'black',
+                         linestyle: str = '-', linewidth: float = 1,
+                         round_start: bool=True, **kwargs):
         """
         Plots the time series of a scalar parameter
 
@@ -521,6 +557,19 @@ class RTP():
             along side the number. Otherwise the cp ID will
             just be printed. This is only used for the parameter cp
             Default: True
+        round_start: bool=True
+                option to round the start time to give tick at start of xaxis
+                Set True to round, set False to plot from start of data.
+                Default: True
+        color: str
+            color of the line
+            default: black
+        linestyle: str
+            style of line with dash marks or solid
+            default: solid
+        linewidth: float
+            the width of the line
+            default: 1
         kwargs
             kwargs passed into plot_date
 
@@ -558,13 +607,11 @@ class RTP():
         try:
             # because of partial records we need to find the first
             # record that has that parameter
-            index_first_match = next(i for i, d in enumerate(dmap_data)
-                                     if parameter in d)
+            next(i for i, d in enumerate(dmap_data) if parameter in d)
         except StopIteration:
             raise plot_exceptions.UnknownParameterError(parameter)
 
         cls.dmap_data = dmap_data
-        check_data_type(cls.dmap_data, parameter, 'scalar', index_first_match)
         start_time, end_time = cls.__determine_start_end_time(start_time,
                                                               end_time)
 
@@ -641,11 +688,23 @@ class RTP():
                        (channel == dmap_record['channel'] or channel == 'all'):
                         # construct the x-axis array
                         x.append(rec_time)
-                        if parameter == 'tfreq':
-                            # Convert kHz to MHz by dividing by 1000
-                            y.append(dmap_record[parameter]/1000)
-                        else:
-                            y.append(dmap_record[parameter])
+                        try:
+                            if parameter == 'tfreq':
+                                # Convert kHz to MHz by dividing by 1000
+                                y.append(dmap_record[parameter]/1000)
+                            elif isinstance(dmap_record[parameter],
+                                            np.ndarray):
+                                if gate in dmap_record['slist']:
+                                    for i in range(len(dmap_record['slist'])):
+                                        if dmap_record['slist'][i] == gate:
+                                            break
+                                    y.append(dmap_record[parameter][i])
+                                else:
+                                    y.append(np.ma.masked)
+                            else:
+                                y.append(dmap_record[parameter])
+                        except KeyError:
+                            y.append(np.ma.masked)
                     # else plot missing data
                     elif len(x) > 0:
                         diff_time = rec_time - x[-1]
@@ -668,13 +727,29 @@ class RTP():
             # otherwise the lines will connect in gapped data
             my = np.ma.array(y)
             my = np.ma.masked_where(np.isnan(my), my)
+
             lines = ax.plot_date(x, my, fmt='k', tz=None, xdate=True,
-                                 ydate=False,
-                                 **kwargs)
-            rounded_down_start_time = x[0] -\
-                timedelta(minutes=x[0].minute % 15,
-                          seconds=x[0].second,
-                          microseconds=x[0].microsecond)
+                                 ydate=False, color=color, linestyle=linestyle,
+                                 linewidth=linewidth)
+
+            if round_start:
+                major_locator, _ = plt.xticks()
+                dt = dates.num2date(major_locator[1]) -\
+                    dates.num2date(major_locator[0])
+                tick_sep = dt.seconds//60
+                if tick_sep > 0:
+                    rounded_down_start_time = x[0] -\
+                        timedelta(minutes=x[0].minute % tick_sep,
+                            seconds=x[0].second,
+                            microseconds=x[0].microsecond)
+                else:
+                    rounded_down_start_time = x[0] -\
+                        timedelta(minutes=x[0].minute % 15,
+                            seconds=x[0].second,
+                            microseconds=x[0].microsecond)
+            else:
+                rounded_down_start_time = x[0]
+
             ax.set_xlim([rounded_down_start_time, x[-1]])
             ax.set_yscale(scale)
 
@@ -682,10 +757,24 @@ class RTP():
         # Rounded the time down to show origin label upon
         # Daniel Billet and others request.
         # TODO: may move this to its own function
-        rounded_down_start_time = x[0] -\
-            timedelta(minutes=x[0].minute % 15,
-                      seconds=x[0].second,
-                      microseconds=x[0].microsecond)
+        if round_start:
+            major_locator, _ = plt.xticks()
+            dt = dates.num2date(major_locator[1]) -\
+                dates.num2date(major_locator[0])
+            tick_sep = dt.seconds//60
+            if tick_sep > 0:
+                rounded_down_start_time = x[0] -\
+                    timedelta(minutes=x[0].minute % tick_sep,
+                          seconds=x[0].second,
+                          microseconds=x[0].microsecond)
+            else:
+                rounded_down_start_time = x[0] -\
+                    timedelta(minutes=x[0].minute % 15,
+                          seconds=x[0].second,
+                          microseconds=x[0].microsecond)
+        else:
+            rounded_down_start_time = x[0]
+
         ax.set_xlim([rounded_down_start_time, x[-1]])
 
         ax.xaxis.set_major_formatter(dates.DateFormatter(date_fmt))
@@ -700,7 +789,10 @@ class RTP():
             tick_interval = 30
         else:
             tick_interval = 1
-        ax.xaxis.set_minor_locator(dates.MinuteLocator(interval=tick_interval))
+        # byminute keyword makes sure that the ticks are situated at
+        # the minute or half hour marks, rather than at a set interval
+        ax.xaxis.set_minor_locator(
+            dates.MinuteLocator(byminute=range(0,60,tick_interval)))
 
         ax.margins(x=0)
         ax.tick_params(axis='y', which='minor')
@@ -709,14 +801,14 @@ class RTP():
         return lines, x, y
 
     @classmethod
-    def plot_summary(cls, dmap_data: List[dict], beam_num: int = 0,
-                     groundscatter: bool = True, channel: int = 'all',
-                     coords: object = Coords.SLANT_RANGE,
-                     reflection_height: float = 250.0,
-                     figsize: tuple = (11, 8.5),
+    def plot_summary(cls, dmap_data: List[dict],
+                     beam_num: int = 0, figsize: tuple = (11, 8.5),
                      watermark: bool = True, boundary: dict = {},
-                     background_color: str = 'w', cmaps: dict = {},
-                     lines: dict = {}, plot_elv: bool = True, title=None):
+                     cmaps: dict = {}, lines: dict = {},
+                     plot_elv: bool = True, title=None,
+                     background: str = 'w', groundscatter: bool = True,
+                     channel: int = 'all', line_color: dict = {},
+                     coords: object = Coords.SLANT_RANGE, **kwargs):
         """
         Plots the summary of several SuperDARN parameters using time-series and
         range-time plots. Please see Notes for further description
@@ -737,6 +829,9 @@ class RTP():
         ax: matplotlib.axes
             axes object for another way of plotting
             Default: None
+        background: string
+            background color of the plots
+            default: white
         groundscatter : boolean
             Flag to indicate if groundscatter should be plotted.
             Placed only on the velocity plot.
@@ -745,13 +840,9 @@ class RTP():
             channel number that will be plotted
             in the summary plot.
             Default: 'all'
-        coords: Coords
+        coords: Coord object
             set the y-axis to a desired coordinate system
-            Default: Coords.SLANT_RANGE
-        reflection_height: float
-            set the ionosphere virtual reflection height
-            this only applies to Coords.GROUND_SCATTER_MAPPED_RANGE
-            Default: 250 (km)
+            Default: Coord.SLANT_RANGE
         figsize : (int,int)
             tuple containing (height, width) figure size
             Default: 11 x 8.5
@@ -770,9 +861,6 @@ class RTP():
         lines: dict or str
             dictionary of time-series line colors.
             Default: black
-        background_color: str
-            changes the color of the background in the plots.
-            Default: white
         boundary: dict
             tuple as the value (zmin, zmax) for the plots
             Default: {'noise.sky': (1e0, 1e5),
@@ -792,7 +880,9 @@ class RTP():
             Default: auto-generated by the files details
             {radar name} {Radar system (if applicable)} Fitacf {version}
             {start hour/date} - {end hour/date}  Beam {number}
-
+        kwargs:
+            reflection_height for ground_scatter_mapped method
+            background
         Raises
         ------
         IndexError
@@ -844,10 +934,19 @@ class RTP():
         boundary_ranges.update(boundary)
 
         # Default color maps for the summary plot
-        line = {'noise.search': 'k',
-                'noise.sky': 'k',
-                'tfreq': 'k',
-                'nave': 'k'}
+        line = {'noise.search': '--',
+                'noise.sky': '-',
+                'tfreq': '--',
+                'nave': '-'}
+        color = {'noise.search': 'k',
+                 'noise.sky': 'k',
+                 'tfreq': 'k',
+                 'nave': 'k'}
+
+        if isinstance(line_color, dict):
+            color.update(line_color)
+        else:
+            color.update({k: lines for k, v in line_color.items()})
 
         if isinstance(lines, dict):
             line.update(lines)
@@ -924,15 +1023,17 @@ class RTP():
                         warnings.simplefilter("ignore")
                         cls.plot_time_series(dmap_data, beam_num=beam_num,
                                              parameter=axes_parameters[i][0],
-                                             channel=channel, scale=scale,
-                                             color=line[axes_parameters[i][0]],
-                                             ax=axes[i], linestyle='-',
-                                             label=labels[i][0])
+                                             scale=scale, channel=channel,
+                                             color=color[axes_parameters[i][0]],
+                                             ax=axes[i],
+                                             linestyle=line[axes_parameters[i][0]],
+                                             label=labels[i][0], **kwargs)
                     axes[i].set_ylabel(labels[i][0], rotation=0, labelpad=30)
                     axes[i].\
                         axhline(y=boundary_ranges[axes_parameters[i][0]][0] +
                                 0.8, xmin=-0.11, xmax=-0.05, clip_on=False,
-                                color=line[axes_parameters[i][0]])
+                                color=color[axes_parameters[i][0]],
+                                linestyle=line[axes_parameters[i][0]])
                     axes[i].set_ylim(boundary_ranges[axes_parameters[i][0]][0],
                                      boundary_ranges[axes_parameters[i][0]][1])
                     # For better y-axis ticks
@@ -949,6 +1050,7 @@ class RTP():
                     if i == 1:
                         # plot the shared parameter
                         second_ax = axes[i].twinx()
+
                         # with warning catch, catches all the warnings
                         # that would be produced by time-series this would be
                         # the citing warning.
@@ -957,18 +1059,19 @@ class RTP():
                             warnings.simplefilter("ignore")
                             cls.plot_time_series(dmap_data, beam_num=beam_num,
                                                  parameter=axes_parameters[i][1],
-                                                 color=line[axes_parameters[i][1]],
+                                                 color=color[axes_parameters[i][1]],
                                                  channel=channel,
                                                  scale=scale, ax=second_ax,
-                                                 linestyle='--')
+                                                 linestyle=line[axes_parameters[i][1]], **kwargs)
                         second_ax.set_xticklabels([])
                         second_ax.set_ylabel(labels[i][1], rotation=0,
                                              labelpad=25)
                         second_ax.\
                             axhline(y=boundary_ranges[axes_parameters[i][1]][0]
                                     + 0.8, xmin=1.07, xmax=1.13,
-                                    clip_on=False, linestyle='--',
-                                    color=line[axes_parameters[i][1]])
+                                    clip_on=False,
+                                    linestyle=line[axes_parameters[i][1]],
+                                    color=color[axes_parameters[i][1]])
                         second_ax.\
                             set_ylim(boundary_ranges[axes_parameters[i][1]][0],
                                      boundary_ranges[axes_parameters[i][1]][1])
@@ -984,7 +1087,7 @@ class RTP():
                                                       MaxNLocator(integer=True,
                                                                   nbins=3))
 
-                axes[i].set_facecolor(background_color)
+                axes[i].set_facecolor(background)
             # plot cp id
             elif i == 2:
                 # with warning catch, catches all the warnings
@@ -993,12 +1096,12 @@ class RTP():
                 with warnings.catch_warnings():
                     warnings.simplefilter("ignore")
                     cls.plot_time_series(dmap_data, beam_num=beam_num,
-                                         parameter=axes_parameters[i],
                                          channel=channel,
-                                         ax=axes[i])
+                                         parameter=axes_parameters[i],
+                                         ax=axes[i], **kwargs)
                 axes[i].set_ylabel('CPID', rotation=0, labelpad=30)
                 axes[i].yaxis.set_label_coords(-0.08, 0.079)
-                axes[i].set_facecolor(background_color)
+                axes[i].set_facecolor(background)
             # plot range-time
             else:
                 # Current standard is to only have groundscatter
@@ -1019,21 +1122,17 @@ class RTP():
                 with warnings.catch_warnings():
                     warnings.simplefilter("ignore")
                     _, cbar, _, x, _, _ =\
-                        cls.plot_range_time(dmap_data,
-                                            beam_num=beam_num,
+                        cls.plot_range_time(dmap_data, beam_num=beam_num,
                                             colorbar_label=labels[i],
-                                            parameter=axes_parameters[i],
-                                            ax=axes[i],
-                                            groundscatter=grndflg,
                                             channel=channel,
-                                            coords=coords,
-                                            reflection_height = reflection_height,
+                                            parameter=axes_parameters[i],
+                                            ax=axes[i], groundscatter=grndflg,
                                             cmap=cmap[axes_parameters[i]],
                                             zmin=boundary_ranges[axes_parameters[i]][0],
                                             zmax=boundary_ranges[axes_parameters[i]][1],
-                                            ymax=ymax,
-                                            yspacing=500,
-                                            background=background_color)
+                                            ymax=ymax, yspacing=500,
+                                            background=background,
+                                            coords=coords, **kwargs)
                 # Overwriting velocity ticks to get a better pleasing
                 # look on the colorbar
                 # Preference by Marina Schmidt
